-- Henge Counter handlers
local AceOO = AceLibrary("AceOO-2.0")
local PT = AceLibrary("PeriodicTable-3.0")
local Metrognome = AceLibrary("Metrognome-2.0")

local counterTable = {}

HengeCounter = AceOO.Class()
HengeCounter.virtual = true

local HengeCounterSpell = AceOO.Class(HengeCounter)
local HengeCounterItem = AceOO.Class(HengeCounter)

		---------------------------------
		-- HengeCounter main functions --
		---------------------------------

--[[ HengeCounter:Add(arg) - Adds the counter defined by 'arg' to the list of
	available counters for use by buttons/spheres/etc.
		'arg' - Must be a _HengeCounter definition table_
]]--
function HengeCounter:Add(arg)
	if type(arg) ~= "table" or
	type(arg.id) ~= "string" or
	type(arg.type) ~= "string" then
		Henge:Debug("%s:Add() - Invalid argument.", self)
		return
	end
	if counterTable[arg.id] then
		Henge:Debug("%s:Add() - Counter '%s' already exists.", self, arg.id)
		return
	end
	if arg.type == 'spell' then
		if type(arg.spell) ~= "string" then
			Henge:Debug("%s:Add() - Invalid arg.spell value.", self)
			return
		end
		counterTable[arg.id] = HengeCounterSpell:new(arg)
	elseif arg.type == 'item' then
		if type(arg.itemid) ~= "number" then
			Henge:Debug("%s:Add() - Invalid arg.itemid value.", self)
			return
		end
		counterTable[arg.id] = HengeCounterItem:new(arg)
	else
		Henge:Debug("%s:Add() - Type '%s' not recognized.", self, arg.type)
		return
	end
end

--[[ HengeCounter:Get(id) - Returns the HengeCounter specified.
		'id' - Must be the 'id' string of an available counter
]]--
function HengeCounter:Get(id)
	if type(id) ~= "string" then
		Henge:Debug("%s:Get() - Invalid argument.", self)
		return nil
	end
	if not AceOO.inherits(counterTable[id], HengeCounter) then
		Henge:Debug("%s:Get() - Counter '%s' does not exist.", self, id)
		return nil
	end
	return counterTable[id]
end

		----------------------------------------------------
		-- HengeCounter - virtual base class for counters --
		----------------------------------------------------
function HengeCounter:ToString()
	return "HengeCounter"
end

function HengeCounter.prototype:init(arg)
	HengeCounter.super.prototype.init(self)

	self.notify = {}
	self.count = 0
	for k, v in pairs(arg) do
		if (not self[k]) and (k ~= "count") and (k ~= "notify") then
			self[k] = v
		else
			Henge:Debug("HengeCounter:new() - Attempt to overwrite '%s'.", k)
		end
	end
	if type(self.always) ~= "boolean" then self.always = false end
	Metrognome:Register("HengeCounter"..self.id, self.OnUpdate, 2, self)
	Metrognome:Start("HengeCounter"..self.id)
end

function HengeCounter.prototype:OnUpdate()
	for k, v in pairs(self.notify) do
		v(self.count, self.always)
	end
end

function HengeCounter.prototype:AddWatch(id, func)
	if type(func) ~= "function" then
		Henge:Debug("%s:Watch() - Invalid function.", self)
		return
	end
	if type(id) ~= "string" then
		Henge:Debug("%s:Watch() - Must specify watch ID.", self)
		return
	end
	if self.notify[id] then
		Henge:Debug("%s:Watch() - Watch ID '%s' already exists.", self, id)
		return
	end
	self.notify[id] = func
end

function HengeCounter.prototype:RemoveWatch(id)
	self.notify[id] = nil
end

function HengeCounter.prototype:GetCount()
	return self.count, self.always
end

		-------------------------------------------------
		-- HengeCounterSpell - Handles spell cooldowns --
		-------------------------------------------------
function HengeCounterSpell:ToString()
	return "HengeCounterSpell"
end

function HengeCounterSpell.prototype:init(arg)
	HengeCounterSpell.super.prototype.init(self, arg)
	Henge:LearnSpellFunc(self.id, function() self:OnLearnSpell() end)
	self:OnLearnSpell()
end

function HengeCounterSpell.prototype:OnLearnSpell()
	if self.spellID and self.icon then return end
	for tloop = 1, 180 do
		local name, _, offset, count = GetSpellTabInfo(tloop)
		if not name then
			break
		end
		for sloop = offset + 1, offset + count do
			local sn, sr = GetSpellName(sloop, BOOKTYPE_SPELL)
			if self.spell == sn and ((not self.rank) or sr == self.rank) then
				self.spellID = sloop
				self.icon = GetSpellTexture(sloop, BOOKTYPE_SPELL)
			end
		end
	end
end

function HengeCounterSpell.prototype:OnUpdate()
	if type(self.spellID) ~= "number" then return nil end
	local _, dur, _ = GetSpellCooldown(self.spellID, BOOKTYPE_SPELL)
	self.count = dur
	for k, v in pairs(self.notify) do
		v(self.count, self.always)
	end
end

		-----------------------------------------------------
		-- HengeCounterItem - Handles tracking item counts --
		-----------------------------------------------------
function HengeCounterItem:ToString()
	return "HengeCounterItem"
end

function HengeCounterItem.prototype:init(arg)
	HengeCounterItem.super.prototype.init(self, arg)
	self.itemID = arg.itemid
end

function HengeCounterItem.prototype:OnUpdate()
	self.count = GetItemCount(self.itemID) or 0
	for k, v in pairs(self.notify) do
		v(self.count, self.always)
	end
end
